/**
 * This class is used to create an CorpusAnalysisClient object.
 * The client handels the client server communication.
 * It requests data (e.g. the analysis session or query results) from the
 * the server and recieves them.
 */
class CorpusAnalysisClient {
  constructor(corpusId, socket, logging=false) {
    this.corpusId = corpusId;
    this.displays = {};
    this.logging = logging;
    this.socket = socket;
    this.socketEventListeners = {};
  }

  // Registers one or more SocketEventListeners to the CorpusAnalysisClient.
  setSocketEventListeners(socketEventListeners) {
    for (let listener of socketEventListeners) {
      this.socketEventListeners[listener.type] = listener.listenerFunction;
    }
  }

  loadSocketEventListeners() {
    for (let [type, listener] of Object.entries(this.socketEventListeners)) {
      listener(this);
    }
  }

  // Registers a CorpusAnalysisDisplay object to the CorpusAnalysisClient.
  setDisplay(type, corpusAnalysisDisplay) {
    this.displays[type] = corpusAnalysisDisplay;
  }

  /**
   * Requests a corpus analysis session via socket.io.
   * Opens a loading modal at the start of the request
   * Should be a private method if ES2020 is finalized (Maybe?)
   */
  requestSession() {
    console.info('corpus_analysis_init: Client requesting session via',
                 'socket.emit');
    if (this.displays.init != undefined) {
      this.displays.init.element.M_Modal.open();
      this.displays.init.setVisibilityByStatus('waiting');
    }
    this.socket.emit('corpus_analysis_init', this.corpusId);
  }

  /**
   * Sends the query string to the server.
   * Should be a private method if ES2020 is finalized (Maybe?)
   */
  requestQueryData(queryStr) {
    console.info('corpus_analysis_query: Client requesting query data via',
                 'socket.emit for the query', queryStr);
    // TODO: Display stuff
    this.socket.emit('corpus_analysis_query', queryStr);
  }
}

/**
 * This class is used to create an CorpusAnalysisDisplay object.
 * Input is one HTMLElement that can then be hidden or shown depending on
 * its CSS classes.
 */
class CorpusAnalysisDisplay {
  constructor(element) {
    // with this function initalized modals can also be handeld
    this.element = (() => {if (element instanceof HTMLElement) {
                            return element;
                          } else {
                            element = element['$el'][0];
                            return element;
                          }
    })();
    this.errorContainer = element.querySelector('.error-container');
    this.showOnError = element.querySelectorAll('.show-on-error');
    this.showOnSuccess = element.querySelectorAll('.show-on-success');
    this.showWhileWaiting = element.querySelectorAll('.show-while-waiting');
    this.hideOnComplete = element.querySelectorAll('.hide-on-complete')
  }

  // Changes the visibility of its own
  setVisibilityByStatus(status) {
    switch (status) {
      case 'error':
        for (let element of this.showOnError) {
          element.classList.remove('hide');
        }
        for (let element of this.showOnSuccess) {
          element.classList.add('hide');
        }
        for (let element of this.showWhileWaiting) {
          element.classList.add('hide');
        }
        break;
      case 'success':
        for (let element of this.showOnError) {
          element.classList.add('hide');

        }
        for (let element of this.showOnSuccess) {
          element.classList.remove('hide');
        }
        for (let element of this.showWhileWaiting) {
          element.classList.add('hide');
        }
        break;
      case 'waiting':
        for (let element of this.showOnError) {
          element.classList.add('hide');
        }
        for (let element of this.showOnSuccess) {
          element.classList.add('hide');
        }
        for (let element of this.showWhileWaiting) {
          element.classList.remove('hide');
        }
        break;
      default:
        // Hide all
        for (let element of this.showOnError) {
          element.classList.add('hide');
        }
        for (let element of this.showOnSuccess) {
          element.classList.add('hide');
        }
        for (let element of this.showWhileWaiting) {
          element.classList.add('hide');
        }
    }
  }
}

/**
 * This class is used to create an CorpusAnalysisDisplay object.
 * Input is one HTMLElement that can then be hidden or shown depending on
 * its CSS classes.
 */
class SocketEventListener {
  constructor(type, listenerFunction) {
    this.listenerFunction = listenerFunction;
    this.type = type;
  }
}

// export Classes from this module
export {CorpusAnalysisClient, CorpusAnalysisDisplay, SocketEventListener};